unit AddDirectProxyDlg;

interface

uses
  Winapi.Windows,
  Winapi.Messages,
  System.SysUtils,
  System.Variants,
  System.Classes,
  Vcl.Graphics,
  Vcl.Controls,
  Vcl.Forms,
  Vcl.Dialogs,
  Vcl.StdCtrls,
  Codec,
  VaxServerCOM,
  VaxTeleServerSIP;

type
  TAddDirectProxyForm = class(TForm)
    Label1: TLabel;
    ComboProxy: TComboBox;
    GroupBox1: TGroupBox;
    Label2: TLabel;
    Label3: TLabel;
    EditAuthUser: TEdit;
    EditPassword: TEdit;
    Label4: TLabel;
    Label5: TLabel;
    EditDomainRealm: TEdit;
    EditProxyIP: TEdit;
    Label6: TLabel;
    EditProxyPort: TEdit;
    GroupBox2: TGroupBox;
    Label7: TLabel;
    ListBoxEnableCodecs: TListBox;
    Label8: TLabel;
    ListBoxDisableCodecs: TListBox;
    btnDisableCodec: TButton;
    btnEnableCodec: TButton;
    GroupBox3: TGroupBox;
    RadioRejectCall: TRadioButton;
    RadioConnectToUser: TRadioButton;
    RadioPutInConf: TRadioButton;
    ComboConnectToUser: TComboBox;
    ComboPutInConf: TComboBox;
    btnAddProxy: TButton;
    btnRemoveProxy: TButton;
    btnClear: TButton;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    rbtnQueue: TRadioButton;
    comboQueueName: TComboBox;
    rbtnRingGroup: TRadioButton;
    comboRingGroup: TComboBox;

    procedure RemoveProxy_Click(Sender: TObject);
    procedure ClearProxyDlg_Click(Sender: TObject);

    procedure ComboProxy_SelIndexChanged(Sender: TObject);

    procedure DisableCodec_Click(Sender: TObject);
    procedure EnableCodec_Click(Sender: TObject);

    procedure OnCreate(Sender: TObject);
    procedure AddProxy_Click(Sender: TObject);

    private

    procedure ShowUsers();
    procedure ShowConfs();
    procedure ShowDirectProxies();
    procedure ShowQueue();
    procedure ShowRingGroup();

    procedure LoadCodecs();

    function AddProxyChecks(var sInCallTo : string; var lInCallType : long) : boolean;

    procedure ClearDlgContent();
    procedure UpdateDlgContent(sAuthUser, sAuthPwd, sRealm, sProxySIP, sCodecList : string; lProxyPort : long; nToPeerType : integer; sToPeerName : string; bBlocked : boolean; sRoomName : string; sQueueName : string; sRingGroupName : string);
    procedure UpdateIncomingCallSection(nToPeerType : integer; sToPeerName : string; bRejected : boolean; sRoomName : string; sQueueName : string; sRingGroupName : string);

    public
    constructor Create(AOwner : TComponent; var pVaxTeleSIP : TVaxTeleServerSIP); reintroduce; overload;

  public

    procedure OnSetActive();

  end;

var
    AddDirectProxyForm: TAddDirectProxyForm;
    m_objCodecs : TCodec;
    m_pVaxTeleSIP : TVaxTeleServerSIP;

implementation

{$R *.dfm}

    constructor TAddDirectProxyForm.Create(AOwner : TComponent; var pVaxTeleSIP : TVaxTeleServerSIP);
    begin

        inherited Create(AOwner);
        m_pVaxTeleSIP := pVaxTeleSIP;

    end;

// events

procedure TAddDirectProxyForm.ClearProxyDlg_Click(Sender: TObject);
begin
    ClearDlgContent();
end;

procedure TAddDirectProxyForm.ComboProxy_SelIndexChanged(Sender: TObject);

var

  sProxyName, sAuthUser, sAuthPwd : string;
  sDomainRealm, sProxySIP : string;
  sCodecList, sRoomName, sToPeerName, sRingGroupName, sQueueName : string;

  bBlocked: boolean ;

  lProxyPort, nToPeerType : integer ;


begin

    if(ComboProxy.Items.Count <= 0) then
        Exit;

    sAuthUser := '';
    sAuthPwd := '';

    sDomainRealm := '';
    sProxySIP := '';
    sCodecList := '';
    sRoomName := '';
    sToPeerName := '';

    bBlocked := false ;

    lProxyPort := -1;
    nToPeerType := -1;

    sProxyName := ComboProxy.Items[ComboProxy.ItemIndex];

    m_pVaxTeleSIP.GetDirectProxyDetails(ComboProxy.Text, sAuthUser, sAuthPwd, sDomainRealm, sProxySIP, lProxyPort, sCodecList, nToPeerType, sToPeerName, bBlocked, sRoomName, sRingGroupName, sQueueName);
    UpdateDlgContent(sAuthUser, sAuthPwd, sDomainRealm, sProxySIP, sCodecList, lProxyPort, nToPeerType, sToPeerName, bBlocked, sRoomName, sQueueName, sRingGroupName);

end;

procedure TAddDirectProxyForm.DisableCodec_Click(Sender: TObject);
begin
    if(ListBoxEnableCodecs.Items.Count <= 0) then
        exit;

    if(ListBoxEnableCodecs.ItemIndex = -1) then
    begin
        ShowMessage('Please choose a codec first');
        exit;
    end;

    ListBoxDisableCodecs.Items.Add(ListBoxEnableCodecs.Items[ListBoxEnableCodecs.ItemIndex]);
    ListBoxEnableCodecs.Items.Delete(ListBoxEnableCodecs.ItemIndex);
end;

procedure TAddDirectProxyForm.EnableCodec_Click(Sender: TObject);
begin
    if(ListBoxDisableCodecs.Items.Count <= 0) then
        exit;

    if(ListBoxDisableCodecs.ItemIndex = -1) then
    begin
        ShowMessage('Please choose a codec first');
        exit;
    end;

    ListBoxEnableCodecs.Items.Add(ListBoxDisableCodecs.Items[ListBoxDisableCodecs.ItemIndex]);
    ListBoxDisableCodecs.Items.Delete(ListBoxDisableCodecs.ItemIndex);

end;

procedure TAddDirectProxyForm.RemoveProxy_Click(Sender: TObject);

var
nIndex : integer;

begin

    if(Length(ComboProxy.Text ) = 0 ) then
    begin
        ShowMessage('Please enter proxy name.');
        exit;
    end;

    m_pVaxTeleSIP.RemoveDirectProxySIP(ComboProxy.Text );

    nIndex := ComboProxy.Items.IndexOf(ComboProxy.Text);
    if(nIndex = -1) then exit;

    ComboProxy.Items.Delete(nIndex);

    ShowMessage('Direct Proxy remove successfully.');
    ClearDlgContent();

end;

procedure TAddDirectProxyForm.OnCreate(Sender: TObject);
begin
    LoadCodecs();
end;

// functions

procedure TAddDirectProxyForm.ShowUsers();
var
nIndex : integer;
objListUsers : TStringList;

begin

    ComboConnectToUser.Items.Clear();

    objListUsers := TStringList.Create();
    m_pVaxTeleSIP.GetUserAll(objListUsers);

    if(objListUsers.Count <= 0) then
        exit;

    for nIndex := 0 to objListUsers.Count - 1 do
        ComboConnectToUser.Items.Add(objListUsers[nIndex]);

    ComboConnectToUser.ItemIndex := 0;
end;

procedure TAddDirectProxyForm.ShowConfs();
var
    objListRooms : TStringList;
    nIndex: Integer;

begin

    objListRooms := TStringList.Create();
    m_pVaxTeleSIP.GetRoomAll(objListRooms);

    if(objListRooms.Count <= 0) then
        exit;

    for nIndex := 0 to objListRooms.Count - 1 do
        ComboPutInConf.Items.Add(objListRooms[nIndex]);
end;

procedure TAddDirectProxyForm.ShowDirectProxies();
var

  sAuthUser, sAuthPwd : string;
  sDomainRealm, sProxySIP : string;
  sCodecList, sRoomName, sToPeerName, sRingGroupName, sQueueName : string;

  bBlocked, bAddToConf : boolean ;

  lProxyPort, nToPeerType : integer ;

  objListProxy : TStringList;


begin
    sAuthUser := '';
    sAuthPwd := '';

    sDomainRealm := '';
    sProxySIP := '';
    sCodecList := '23014';
    sRoomName := '';
    sToPeerName := '';
    sRingGroupName := '';
    sQueueName := '';

    bBlocked := true ;
    bAddToConf := false;

    lProxyPort := 5060;
    nToPeerType := -1;

    objListProxy := TStringList.Create();
    m_pVaxTeleSIP.GetDirectProxyAll(objListProxy);

    if(objListProxy.Count <= 0) then
    begin
        RadioRejectCall.Checked := True;
        //UpdateDlgContent('', '', '192.168.0.102', '192.168.0.102', '23014', 5060, -1, '', True, False, '');
        exit;
    end;

    m_pVaxTeleSIP.GetDirectProxyDetails(ComboProxy.Text, sAuthUser, sAuthPwd, sDomainRealm, sProxySIP, lProxyPort, sCodecList, nToPeerType, sToPeerName, bBlocked, sRoomName, sRingGroupName, sQueueName);
    UpdateDlgContent(sAuthUser, sAuthPwd, sDomainRealm, sProxySIP, sCodecList, lProxyPort, nToPeerType, sToPeerName, bBlocked, sRoomName, sQueueName, sRingGroupName);

end;


procedure TAddDirectProxyForm.ShowQueue();
var
    objListQueue : TStringList;
    nIndex: Integer;

begin

    objListQueue := TStringList.Create();
    m_pVaxTeleSIP.GetQueueList(objListQueue);

    if(objListQueue.Count <= 0) then
        exit;

    comboQueueName.Clear();

    for nIndex := 0 to objListQueue.Count - 1 do
        comboQueueName.Items.Add(objListQueue[nIndex]);



    if(comboQueueName.Items.Count > 0) Then
        comboQueueName.ItemIndex := 0;

end;

procedure TAddDirectProxyForm.ShowRingGroup();
var
    objListRingGroup : TStringList;
    nIndex: Integer;

begin

    objListRingGroup := TStringList.Create();
    m_pVaxTeleSIP.GetRingGroupList(objListRingGroup);

    if(objListRingGroup.Count <= 0) then
        exit;

    comboRingGroup.Clear();

    for nIndex := 0 to objListRingGroup.Count - 1 do
        comboRingGroup.Items.Add(objListRingGroup[nIndex]);


    if(comboRingGroup.Items.Count > 0) Then
        comboRingGroup.ItemIndex := 0;
end;



procedure TAddDirectProxyForm.OnSetActive();
begin
    ShowUsers();
    ShowConfs();
    ShowDirectProxies();
    ShowQueue();
    ShowRingGroup();

end;

procedure TAddDirectProxyForm.LoadCodecs();

var
  nIndex : Integer;

begin

    for nIndex := 0 to m_objCodecs.TOTAL_SIZE_CODEC - 1  do
    begin
        ListBoxEnableCodecs.Items.Add(m_objCodecs.GetCodecName(IntToStr(nIndex)));
    end;
end;

function TAddDirectProxyForm.AddProxyChecks(var sInCallTo : string; var lInCallType : long) : boolean;
var
nIndex : integer;

begin

    if(RadioRejectCall.Checked = true) then
        RadioRejectCall.Checked := true;

    if(RadioConnectToUser.Checked = true) then
    begin
        nIndex := ComboConnectToUser.ItemIndex;
        if(nIndex = -1) then
        begin
            ShowMessage('Please select the user/login to forward the incoming call.');
            exit;
        end;

        lInCallType := TVaxServerCOM.VAX_PEER_TYPE_USER;
        sInCallTo := ComboConnectToUser.Text;
    end;

    if(RadioPutInConf.Checked = true) then
    begin

        nIndex := ComboPutInConf.ItemIndex;

        if(nIndex = -1) then
        begin

            ShowMessage('Please select the room to forward the incoming call.');
            exit;

        end;

        sInCallTo := ComboPutInConf.Text;

    end;

    Result := True;

end;

procedure TAddDirectProxyForm.AddProxy_Click(Sender: TObject);
var
sInCallTo, sCodecList, sCodecName, sCodecNo, sToPeerName, sRoomName, sQueueName, sRingGroupName: string;
lInCallType, nIndex, nToPeerType : integer;
bRejected, bAddToConf : boolean;

begin

    sInCallTo := '';sCodecList :=''; sCodecName:='';
    sCodecNo:=''; sToPeerName:='';sRoomName:='';

    bRejected:=false; bAddToConf:=false;

    lInCallType := -1;

    if(AddProxyChecks(sInCallTo, lInCallType) = false) then
        exit;

    if(ComboProxy.Items.IndexOf(ComboProxy.Text ) <> -1) then
    begin
        ShowMessage('Proxy already exists.');
        exit;
    end;

    sCodecList := '';
    sCodecName := '';
    for nIndex := 0 to ListBoxEnableCodecs.Items.Count - 1 do
    begin
        sCodecName := ListBoxEnableCodecs.Items[nIndex];    // get codec name
        sCodecNo := m_objCodecs.GetCodecNo(sCodecName);  // use codec name and get codec no
        sCodecList := sCodecList + sCodecNo;     // concat all these codec numbers
    end;

    if(RadioRejectCall.Checked = true) then
        bRejected := true;

    if(RadioConnectToUser.Checked = true) then
    begin
        nToPeerType := TVaxServerCOM.VAX_PEER_TYPE_USER;
        sToPeerName := ComboConnectToUser.Text;
    end;

    if(RadioPutInConf.Checked = true) then
    begin
        sRoomName := ComboPutInConf.Text;
        bAddToConf := true;
    end;

    if(rbtnQueue.Checked = true) then
    begin
        sQueueName:= comboQueueName.Text;
    end;

    if(rbtnRingGroup.Checked = true) then
    begin
        sRingGroupName := comboRingGroup.Text;
    end;

    if (m_pVaxTeleSIP.AddDirectProxySIP(ComboProxy.Text, EditAuthUser.Text, EditPassword.Text, EditDomainRealm.Text, EditProxyIP.Text, StrToInt(EditProxyPort.Text), sCodecList, nToPeerType, sToPeerName, bRejected, sRoomName, sRingGroupName, sQueueName) = false) then
    begin
        ShowMessage('Proxy could not be added.');
        exit;
    end;

    nIndex := ComboProxy.Items.Add(ComboProxy.Text );
    ComboProxy.ItemIndex := nIndex;

    ShowMessage('Proxy Added successfully.');


end;

procedure TAddDirectProxyForm.ClearDlgContent();
begin

    ComboProxy.Text := '';

    EditAuthUser.Text := '';
    EditPassword.Text := '';

    EditDomainRealm.Text := '';
    EditProxyIP.Text := '';
    EditProxyPort.Text := '5060';

    if(ComboConnectToUser.Items.Count > 0) then
        ComboConnectToUser.ItemIndex := 0;

    if(ComboPutInConf.Items.Count > 0) then
        ComboPutInConf.ItemIndex := 0;

    if(comboQueueName.Items.Count > 0) then
        comboQueueName.ItemIndex := 0;

    if(comboRingGroup.Items.Count > 0) then
        comboRingGroup.ItemIndex := 0;

    RadioRejectCall.Checked := true;

    ListBoxEnableCodecs.Items.Clear();
    ListBoxDisableCodecs.Items.Clear();

    LoadCodecs();

end;
procedure TAddDirectProxyForm.UpdateDlgContent(sAuthUser, sAuthPwd, sRealm, sProxySIP, sCodecList : string; lProxyPort : long; nToPeerType : integer; sToPeerName : string; bBlocked : boolean; sRoomName : string; sQueueName : string; sRingGroupName : string);

var
  nIndex: Integer;
  sCodecName: string;

begin
    EditAuthUser.Text := sAuthUser;
    EditPassword.Text := sAuthPwd;
    EditDomainRealm.Text := sRealm;
    EditProxyIP.Text := sProxySIP;
    EditProxyPort.Text := IntToStr(lProxyPort);

    UpdateIncomingCallSection(nToPeerType, sToPeerName, bBlocked, sRoomName, sQueueName, sRingGroupName);

    ListBoxEnableCodecs.Items.Clear();
    ListBoxDisableCodecs.Items.Clear();

    for nIndex := 0 to m_objCodecs.TOTAL_SIZE_CODEC -1 do
        ListBoxDisableCodecs.Items.Add(m_objCodecs.GetCodecName(IntToStr(nIndex)));

    for nIndex := 0 to Length(sCodecList) -1 do
    begin
        sCodecName := m_objCodecs.GetCodecName(sCodecList[nIndex+1]);
        ListBoxEnableCodecs.Items.Add(sCodecName);
        ListBoxDisableCodecs.Items.Delete(ListBoxDisableCodecs.Items.IndexOf(sCodecName));
    end;


end;
procedure TAddDirectProxyForm.UpdateIncomingCallSection(nToPeerType : integer; sToPeerName : string; bRejected : boolean; sRoomName : string; sQueueName : string; sRingGroupName : string);

var
nIndex : integer;

begin

    if(bRejected = true ) then
        RadioRejectCall.Checked := true;

    if(nToPeerType = TVaxServerCOM.VAX_PEER_TYPE_USER) then
    begin
        nIndex := ComboConnectToUser.Items.IndexOf(sToPeerName);
        if(nIndex = -1) then
        begin
            RadioRejectCall.Checked := true;
            exit;
        end;

        ComboConnectToUser.ItemIndex := nIndex;
        RadioConnectToUser.Checked := true;

    end;

    if(Length(sRoomName) <> 0) then
    begin

        nIndex := ComboPutInConf.Items.IndexOf(sRoomName);

        if(nIndex = -1) then
        begin

            RadioRejectCall.Checked := true;
            exit;

        end;

        ComboPutInConf.ItemIndex := nIndex;
        RadioPutInConf.Checked := true;

    end;

       if(sQueueName <> '') then
    begin
        nIndex := comboQueueName.Items.IndexOf(sQueueName);
        comboQueueName.ItemIndex := nIndex;
        rBtnQueue.Checked := True;
    end;

    if(sRingGroupName <> '') then
    begin
        nIndex := comboRingGroup.Items.IndexOf(sRingGroupName);
        comboRingGroup.ItemIndex := nIndex;
        rBtnRingGroup.Checked := True;
    end;

end;




end.
