﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace ServerSIP
{
    public class cVaxServerExCOM : cVaxServerCOM
    {
        int m_nDialToneId = -1;
        int m_nMusicCallParkingId = -1;
        
        cVaxCallSessionRoute m_objRouteCallSIP = null;
        cConfRoom m_objConfRoom = null;
        cVaxChat m_objChat = null;
        cVaxQueue m_objQueue = null;
        cVaxRingGroup m_objRingGroup = null;

        cQueueList m_objQueueList = null;
        cRingGroupList m_objRingGroupList = null;
        cCallPickUpList m_objCallPickUpList = null;
        cVaxRegUserList m_objVaxRegUserList = null;

        Boolean m_bInitialized = false;
        
        public static cVaxServerExCOM m_objVaxServerExCOM = null;

        #region Methods

        public cVaxServerExCOM()
        {
            m_objConfRoom = new cConfRoom(this);
            m_objRouteCallSIP = new cVaxCallSessionRoute(this);
            m_objChat = new cVaxChat(this);
            m_objQueue = new cVaxQueue();
            m_objRingGroup = new cVaxRingGroup();

            m_objQueueList = new cQueueList();
            m_objRingGroupList = new cRingGroupList();
            m_objCallPickUpList = new cCallPickUpList();

            m_objVaxRegUserList = new cVaxRegUserList();
            m_objVaxServerExCOM = this;
        }

        ~cVaxServerExCOM()
        {

        }
        
	    public new bool Initialize(string  sDomainRealm, string  sSIPListenIP, int nSIPListenPort, string  sRTPListenIP, int nRTPListenPort)
        {
            UnInitialize();
            
            if(!base.Initialize(sDomainRealm, sSIPListenIP, nSIPListenPort, sRTPListenIP, nRTPListenPort))
                return false;

            m_bInitialized = true;
            OnInitialize();

            return true;
        }
	    
        public new void UnInitialize()
        {
            m_bInitialized = false; 
            DeleteUserDataAll();
            DeleteCallSessionDataAll();

            m_objConfRoom.DeleteRoomAll();
                
            m_objVaxRegUserList.DeleteRegUserAll();

            m_objQueueList.DeleteQueueAll();
            m_objRingGroupList.DeleteRingGroupAll();
            m_objCallPickUpList.DeletePickGroupAll();
                
            base.UnInitialize();
        }

        public bool IsInitialized()
        {
            return m_bInitialized;
        }


        public bool AcceptCallSession(int nSessionId, string sCallerName, string sCallerId, string sDialNo, int nToPeerType, string sToPeerName)
        {
            return base.AcceptCallSession(nSessionId, sCallerName, sCallerId, sDialNo, nToPeerType, sToPeerName, 30);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public bool AddUser(string sUserName, string sPassword, string sCodecList, bool bBlocked, string sUserPrefix, string sDialPrefix, int nToPeerType, string sToPeerName, bool bStealthlisten)
        {
            if(!base.AddUser(sUserName, sPassword, sCodecList))
            {
                return false;
            }

            cVaxUserData objUserData = new cVaxUserData();
            base.SetUserData(sUserName, objUserData);

            objUserData.SetData(sPassword, sCodecList, bBlocked, sUserPrefix, sDialPrefix, nToPeerType, sToPeerName, bStealthlisten);

            return true;
        }

        public new void RemoveUser(string sUserName)
        {
            if(!IsUserExist(sUserName))
                return;

            base.FreeUserData(sUserName);
            base.RemoveUser(sUserName);
        }
        
        private bool IsUserExist(string sUserName)
        {
            if(!base.TrvsUserStart())
                return false;

            while (true)
            {
                string sUser = base.TrvsUserNext();
                if(sUser.Length == 0) break;

                if(sUser == sUserName)
                    return true;
            }

            return false;
        }

        
        public bool IsUserBlocked(string sUserName)
        {
            if(!IsUserExist(sUserName))
                return true;

            cVaxUserData objUserData = (cVaxUserData)base.GetUserData(sUserName);
            if(objUserData == null) return true;

            return objUserData.IsUserBlocked();
        }

        public bool IsUserRegistered(string sUserName)
        {
            if(!IsUserExist(sUserName))
                return false;

            cVaxUserData objUserData = (cVaxUserData)base.GetUserData(sUserName);
            if(objUserData == null) return false;

            return objUserData.IsRegistered();
        }

        public void SetUserRegistered(string sUserName)
        {
            if(!IsUserExist(sUserName))
                return;

            cVaxUserData objUserData = (cVaxUserData)base.GetUserData(sUserName);
            if(objUserData == null) return;

            objUserData.SetUserRegistered();
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public bool AddLine(string sLineName, string sDisplayName, string sUserName, string sAuthUser, string sAuthPwd, string sDomainRealm, string sProxySIP, string sCodecList, bool bRegister, int nToPeerType, string sToPeerName, bool bRejected, string sRoomName, string sQueueName, string sRingGroupName)
        {
            if(!base.AddLine(sLineName, sDisplayName, sUserName, sAuthUser, sAuthPwd, sDomainRealm, sProxySIP, "", sCodecList))
            {
                return false;
            }

            cVaxLine objLine = new cVaxLine(this);

            base.SetLineData(sLineName, objLine);
            objLine.SetData(sDisplayName, sUserName, sAuthUser, sAuthPwd, sDomainRealm, sProxySIP, sCodecList, bRegister, nToPeerType, sToPeerName, bRejected, sRoomName, sQueueName, sRingGroupName);

            if(bRegister)
                base.RegisterLine(sLineName, 1800);

            return true;
        }

        public new void RemoveLine(string sLineName)
        {
            if(!IsLineExist(sLineName))
                return;

            base.FreeLineData(sLineName);
            base.RemoveLine(sLineName);
        }

        private bool IsLineExist(string sLineName)
        {
            if(!base.TrvsLineStart())
                return false;

            while (true)
            {
                string sLine = base.TrvsLineNext();
                if(sLine.Length == 0) break;

                if(sLine == sLineName)
                    return true;
            }

            return false;
       }
       
        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public bool AddDirectProxySIP(string sProxyName, string sAuthLogin, string sAuthPwd, string sDomainRealm, string sProxyIP, int nProxyPort, string sCodecList, int nToPeerType, string sToPeerName, bool bRejected, string sRoomName, string sQueueName, string sRingGroupName)
        {
            if(!base.AddDirectProxySIP(sProxyName, sAuthLogin, sAuthPwd, sDomainRealm, sProxyIP, nProxyPort, sCodecList))
            {
                return false;
            }

            cVaxDirectProxy objDirectProxy = new cVaxDirectProxy(this);
            base.SetDirectProxyData(sProxyName, objDirectProxy);

            objDirectProxy.SetData(sAuthLogin, sAuthPwd, sDomainRealm, sProxyIP, nProxyPort, sCodecList, nToPeerType, sToPeerName, bRejected, sRoomName, sQueueName, sRingGroupName);
            return true;
        }

        public new void RemoveDirectProxySIP(string sProxyName)
        {
            if(!IsDirectProxyExist(sProxyName))
                return;

            base.RemoveDirectProxySIP(sProxyName);
            base.FreeDirectProxyData(sProxyName);
        }

        private bool IsDirectProxyExist(string sProxyName)
        {
            if(!base.TrvsDirectProxyStart())
                return false;

            while (true)
            {
                string sProxy = base.TrvsDirectProxyNext();
                if(sProxy.Length == 0) break;

                if(sProxy == sProxyName)
                    return true;
            }

            return false;
        }
        
        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public new int GetVaxErrorCode()
        {
            return base.GetVaxErrorCode();
        }

        public bool DialToneToCallSession(bool bEnable, int nSessionId)
        {
            if(m_nDialToneId == -1)
                return true;

            return base.DialToneToCallSession(bEnable, nSessionId, m_nDialToneId);
        }

        public new int GetCallCount()
        {
            return base.GetCallCount();
        }

        public int GetRegisteredUserCount()
        {
            return m_objVaxRegUserList.GetRegUserCount();
        }
        
        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public bool AddRoom(string sRoomName, string sRoomNo)
        {
            return m_objConfRoom.AddRoom(sRoomName, sRoomNo);
        }

        public void RemoveRoom(string sRoomName)
        {
            m_objConfRoom.RemoveRoom(sRoomName);
        }

        public bool IsRoomNoExists(string sRoomNo)
        {
            return m_objConfRoom.IsRoomNoExists(sRoomNo);
        }
        
        public string GetRoomName(string sRoomNo)
        {
            return m_objConfRoom.GetRoomName(sRoomNo);
        }

        ///////////////////////////////////////////////////////////////////////////////////
       ///////////////////////////////////////// Queue ////////////////////////////////////

        public bool AddQueue(string sQueueName, string sQueueMusicFile, string sQueueDialToneFile)
        {

            int nQueueMusicId = LoadWaveFile(sQueueMusicFile);
            int nQueueDialToneId = LoadWaveFile(sQueueDialToneFile);

            if(!m_objQueueList.AddQueue(sQueueName, sQueueMusicFile, nQueueMusicId, sQueueDialToneFile, nQueueDialToneId))
                return false;

            return base.AddQueue(sQueueName);
        }

        public new void RemoveQueue(string sQueueName)
        {
            int nQueueMusicId = m_objQueueList.GetQueueMusicId(sQueueName);
            int nQueueDialToneId = m_objQueueList.GetQueueDialToneId(sQueueName);

            UnLoadWaveFile(nQueueMusicId);
            UnLoadWaveFile(nQueueDialToneId);

            m_objQueueList.RemoveQueue(sQueueName);
            base.RemoveQueue(sQueueName);          
        }

        public new bool AddQueueAgent(string sQueueName, string sUserName)
        {
            if(!m_objQueueList.AddQueueAgent(sQueueName, sUserName))
                return false;
                
            return base.AddQueueAgent(sQueueName, sUserName);            
        }

        public new void RemoveQueueAgent(string sQueueName, string sUserName)
        {
            m_objQueueList.RemoveQueueAgent(sQueueName, sUserName);
            base.RemoveQueueAgent(sQueueName, sUserName);        
        }

        public new void SetQueueAgentPriority(string sQueueName, string sUserName, int nPriorityWeight)
        {
            m_objQueueList.SetQueueAgentPriority(sQueueName, sUserName, nPriorityWeight);
            base.SetQueueAgentPriority(sQueueName, sUserName, nPriorityWeight);
        }

        public new void SetQueueProcessMode(string sQueueName, int nProcessMode, int nProcessType)
        {
            m_objQueueList.SetQueueProcessMode(sQueueName, nProcessMode, nProcessType);
            base.SetQueueProcessMode(sQueueName, nProcessMode, nProcessType);           
        }
                
        public int GetQueueMusicId(string sQueueName)
        {
            return m_objQueueList.GetQueueMusicId(sQueueName);
        }

        public int GetQueueDialToneId(string sQueueName)
        {
            return m_objQueueList.GetQueueDialToneId(sQueueName);
        }

        ///////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////// RingGroup ///////////////////////////////////////////////

        public new bool AddRingGroup(string sRingGroupName)
        {
         
            if(!m_objRingGroupList.AddRingGroup(sRingGroupName))
                return true;

            return base.AddRingGroup(sRingGroupName);
        }

        public new void RemoveRingGroup(string sRingGroupName)
        {
            m_objRingGroupList.RemoveRingGroup(sRingGroupName);
            base.RemoveRingGroup(sRingGroupName);            
        }

        public new bool AddRingGroupAgent(string sRingGroupName, string sUserName)
        {
            if(!m_objRingGroupList.AddRingGroupAgent(sRingGroupName, sUserName))
                return false;

            return base.AddRingGroupAgent(sRingGroupName, sUserName);            
        }

        public new void RemoveRingGroupAgent(string sRingGroupName, string sUserName)
        {
            m_objRingGroupList.RemoveRingGroupAgent(sRingGroupName, sUserName);
            base.RemoveRingGroupAgent(sRingGroupName, sUserName);            
        }

        public new void SetRingGroupProcessMode(string sRingGroupName, int nProcessMode, int nModeType)
        {
            m_objRingGroupList.SetRingGroupProcessMode(sRingGroupName, nProcessMode, nModeType);
            base.SetRingGroupProcessMode(sRingGroupName, nProcessMode, nModeType);
        }

        public new bool SetRingGroupAgentPriority(string sRingGroupName, string sUserName, int nPriority)
        {
            if(!m_objRingGroupList.SetRingGroupAgentPriority(sRingGroupName, sUserName, nPriority))
                return false;

            return base.SetRingGroupAgentPriority(sRingGroupName, sUserName, nPriority);            
        }

        ///////////////////////////////////////////////////////////////////////////////////
        /////////////////////////  CallPickup  ///////////////////////////////////////////

        public new bool AddCallPickUpGroup(string sCallPickUpGroupName)
        {
            if(!m_objCallPickUpList.AddCallPickUpGroup(sCallPickUpGroupName))
                return false;

            return base.AddCallPickUpGroup(sCallPickUpGroupName);
        }

        public new void RemoveCallPickUpGroup(string sCallPickUpGroupName)
        {
            m_objCallPickUpList.RemoveCallPickUpGroup(sCallPickUpGroupName);
            base.RemoveCallPickUpGroup(sCallPickUpGroupName);
        }

        public new bool AddCallPickUpGroupMember(string sCallPickUpGroupName, string sMemberName)
        {
            if(!m_objCallPickUpList.AddCallPickUpGroupMember(sCallPickUpGroupName, sMemberName))
                return false;

            return base.AddCallPickUpGroupMember(sCallPickUpGroupName, sMemberName);
        }

        public void RemoveCallPickupMember(string sCallPickUpGroupName, string sMemberName)
        {
            m_objCallPickUpList.RemoveCallPickUpMember(sCallPickUpGroupName, sMemberName);
            base.RemoveCallPickUpGroupMember(sCallPickUpGroupName, sMemberName);
        }

        public new bool PickUpCall(int nSessionId, string sMemberName)
        {
            return base.PickUpCall(nSessionId, sMemberName); 
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public bool LoadMusicDialTone(string sFileName)
        {
            UnLoadMusicDialTone();
            m_nDialToneId = base.LoadWaveFile(sFileName);

            return (m_nDialToneId != -1);
        }

        public void UnLoadMusicDialTone()
        {
            if(m_nDialToneId != -1)
            {
                base.UnLoadWaveFile(m_nDialToneId);
                m_nDialToneId = -1;
            }
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public bool LoadMusicCallParking(string sFileName)
        {
            UnLoadMusicCallParking();
            m_nMusicCallParkingId = base.LoadWaveFile(sFileName);

            return (m_nMusicCallParkingId != -1);
        }

        public void UnLoadMusicCallParking()
        {
            if(m_nMusicCallParkingId != -1)
            {
                base.UnLoadWaveFile(m_nMusicCallParkingId);
                m_nMusicCallParkingId = -1;
            }
        }

        public int GetMusicCallParking()
        {
            return m_nMusicCallParkingId;
        }
                
        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public bool AddUserRoute(string sUserName, string sUserPrefix, string sDialPrefix, int nToPeerType, string sToPeerName)
        {
            return m_objRouteCallSIP.AddUserRoute(sUserName, sUserPrefix, sDialPrefix, nToPeerType, sToPeerName);
        }

        public void RemoveUserRoute(string sUserName)
        {
            m_objRouteCallSIP.RemoveUserRoute(sUserName);
        }

        public bool AddLineRoute(string sLineName, int nToPeerType, string sToPeerName, bool bRejected,string sRoomName, string sQueueName, string sRingGroupName)
        {
            return m_objRouteCallSIP.AddLineRoute(sLineName, nToPeerType, sToPeerName, bRejected, sRoomName, sQueueName, sRingGroupName);
        }

        public void RemoveLineRoute(string sLineName)
        {     
            m_objRouteCallSIP.RemoveLineRoute(sLineName);
        }

        public bool AddDirectProxyRoute(string sProxyName, int nToPeerType, string sToPeerName, bool bRejected, string sRoomName, string sQueueName, string sRingGroupName)
        {
            return m_objRouteCallSIP.AddDirectProxyRoute(sProxyName, nToPeerType, sToPeerName, bRejected, sRoomName, sQueueName, sRingGroupName);
        }

        public void RemoveDirectProxyRoute(string sProxyName)
        {
            m_objRouteCallSIP.RemoveDirectProxyRoute(sProxyName);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////
                
        public void DeleteCallSessionDataAll()
        {
            base.TrvsCallSessionStart();

            for (int nCount = 0; nCount < base.GetCallSessionCount(); nCount++)
            {
                int nSessionId = base.TrvsCallSessionNext();
                    
                cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);

                base.FreeCallSessionData(nSessionId);
            }
        }

        public void DeleteUserDataAll()
        {
            base.TrvsUserStart();

            for (int nCount = 0; nCount < base.GetUserCount(); nCount++)
            {
                string sUserName = base.TrvsUserNext();
                base.FreeUserData(sUserName);
            }
        }

        private void GetFromPeerRouteinfo(int nFromPeerType, string sDialNo, string sFromPeerName, ref  string sToPeerName, ref string sDialToNo, ref string sRoomName, ref string sRingGroupName, ref string sQueueName, ref int nToPeerType, ref bool bRejected)
        {
            if(nFromPeerType == cVaxServerCOM.VAX_PEER_TYPE_USER)
            {
                if(IsRoomNoExists(sDialNo))
                    sRoomName = cVaxServerExCOM.m_objVaxServerExCOM.GetRoomName(sDialNo);
                else
                   GetUserRouteInfo(sFromPeerName, sDialNo, ref sDialToNo, ref nToPeerType, ref sToPeerName);
            }

            else if(nFromPeerType == cVaxServerCOM.VAX_PEER_TYPE_LINE)
            {
                cVaxServerExCOM.m_objVaxServerExCOM.GetLineRouteInfo(sFromPeerName, ref nToPeerType, ref sToPeerName, ref bRejected, ref sRoomName, ref sQueueName, ref sRingGroupName);
                sDialToNo = sToPeerName;
            }

            else if(nFromPeerType == cVaxServerCOM.VAX_PEER_TYPE_DIRECT_PROXY)
            {
                cVaxServerExCOM.m_objVaxServerExCOM.GetDirectProxyRouteInfo(sFromPeerName, ref nToPeerType, ref sToPeerName, ref bRejected, ref sRoomName, ref sQueueName, ref sRingGroupName);
                sDialToNo = sToPeerName;
            }
        }

        public bool GetUserRouteInfo(string sUserName, string sUserDialedNo, ref string sDialNo, ref int nToPeerType, ref string sToPeerName)
        {
            return m_objRouteCallSIP.GetUserRouteInfo(sUserName, sUserDialedNo, ref sDialNo, ref nToPeerType, ref sToPeerName);
        }

        public bool GetLineRouteInfo(string sLineName, ref int nToPeerType, ref string sToPeerName, ref bool bRejected, ref string sRoomName, ref string sQueueName, ref string sRingGroupName)
        {
            return m_objRouteCallSIP.GetLineRouteInfo(sLineName, ref nToPeerType, ref sToPeerName, ref bRejected, ref sRoomName, ref sQueueName, ref sRingGroupName);
        }

        public bool GetDirectProxyRouteInfo(string sProxyName, ref int nToPeerType, ref string sToPeerName, ref bool bRejected, ref string sRoomName, ref string sQueueName, ref string sRingGroupName)
        {
            return m_objRouteCallSIP.GetDirectProxyRouteInfo(sProxyName, ref nToPeerType, ref sToPeerName, ref bRejected, ref sRoomName, ref sQueueName, ref sRingGroupName);
        }

        //////////////////////////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////////////////////////
           
        #endregion

        #region Events

        public override void OnCallSessionCreated(int nSessionId, int nReasonCode)
        {
            cVaxCallSession objCallSession = new cVaxCallSession();
            base.SetCallSessionData(nSessionId, objCallSession);

            objCallSession.OnCallSessionCreated(nSessionId, nReasonCode);
        }

        public override void OnCallSessionClosed(int nSessionId, int nChannelId, int nReasonCode)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);
            if(objCallSession == null) return;
                        
            objCallSession.OnCallSessionClosed(nSessionId, nChannelId, nReasonCode);
            
            base.FreeCallSessionData(nSessionId);
        }

        public override void OnIncomingCall(int nSessionId, string sCallerName, string sCallerId, string sDialNo, int nFromPeerType, string sFromPeerName, string sUserAgentName, string sFromIP, int nFromPort)
        {
            string sToPeerName = "", sDialToNo = "";
            string sRoomName = "";
            string sRingGroupName = "", sQueueName = "";

            int nToPeerType = -1;
            bool bRejected = false;

            GetFromPeerRouteinfo(nFromPeerType, sDialNo, sFromPeerName, ref sToPeerName, ref sDialToNo, ref sRoomName, ref sRingGroupName, ref sQueueName, ref nToPeerType, ref bRejected);

            if (bRejected)
            {
                RejectCallSession(nSessionId, 404, "Not Found");
                return;
            }

            DialToneToCallSession(true, nSessionId);

            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);
            if (objCallSession == null) return;

            objCallSession.OnIncomingCall(nSessionId, sCallerName, sCallerId, sDialNo, sRoomName, sQueueName, sRingGroupName, nFromPeerType, sFromPeerName, nToPeerType, sToPeerName, sUserAgentName, sFromIP, nFromPort);
        }

        public override void OnCallSessionConnecting(int nSessionId, int nChannelId, int nStatusCode, string sReasonPhrase)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);

            if (objCallSession == null)
                return;

            objCallSession.OnCallSessionConnecting(nSessionId, nChannelId, nStatusCode, sReasonPhrase);
        }

        public override void OnCallSessionConnected(int nSessionId)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);

            if(objCallSession == null)
                return;

            objCallSession.OnCallSessionConnected(nSessionId);
        }

        public override void OnCallSessionOnHold(int nSessionId, int nChannelId)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);

            if(objCallSession == null)
                return;
            
            objCallSession.OnCallSessionOnHold(nSessionId, nChannelId);
        }

        public override void OnCallSessionOffHold(int nSessionId, int nChannelId)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);

            if(objCallSession == null)
                return;

            objCallSession.OnCallSessionOffHold(nSessionId, nChannelId);
        }

        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        public override void OnCallSessionTransferBlind(int nTransfererSessionId, int nTransfererChannelId, string sTransferer, string sTransferee, string sTransferTo)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nTransfererSessionId);

            if(objCallSession == null)
                return;

            objCallSession.OnCallSessionTransferBlind(nTransfererSessionId, nTransfererChannelId, sTransferer, sTransferee, sTransferTo);
        }

        public override void OnCallSessionTransferConsult(int nTransfererSessionId, int nTransfererChannelId, int nTransferToSessionId, int nTransferToChannelId, string sTransferer, string sTransferee, string sTransferTo)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nTransfererSessionId);

            if(objCallSession == null)
                return;

            objCallSession.OnCallSessionTransferConsult(nTransfererSessionId, nTransfererChannelId, nTransferToSessionId, nTransferToChannelId, sTransferer, sTransferee, sTransferTo);
        }

        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        public override void OnDetectedDigitDTMF(int nSessionId, int nChannelId, string sDigitDTMF)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);

            if(objCallSession == null)
                return;

            objCallSession.OnDetectedDigitDTMF(nSessionId, nChannelId, sDigitDTMF);
        }
                
        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

        public override void OnLineRegisterTrying(string sLineName)
        {
            cVaxLine objLine = (cVaxLine)base.GetLineData(sLineName);
            objLine.OnLineRegisterTrying(sLineName);
        }
        public override void OnLineRegisterFailed(string sLineName, int nStatusCode, string sReasonPhrase)
        {
            cVaxLine objLine = (cVaxLine)base.GetLineData(sLineName);
            if(objLine == null) return;

            objLine.OnLineRegisterFailed(sLineName, nStatusCode, sReasonPhrase);
        }
        public override void OnLineRegisterSuccess(string sLineName)
        {
            cVaxLine objLine = (cVaxLine)base.GetLineData(sLineName);
            objLine.OnLineRegisterSuccess(sLineName);
        }

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

        public override void OnRegisterUser(string sUserLogin, string sDomain, string sUserAgentName, string sFromIP, int nFromPort, int nRegId)
        {
            cVaxRegUser objVaxRegUser = m_objVaxRegUserList.AddRegUserData(sUserLogin);

            if(objVaxRegUser == null)
                return;

            objVaxRegUser.OnRegisterUser(sUserLogin, sDomain, sUserAgentName, sFromIP, nFromPort, nRegId);
        }

        public override void OnRegisterUserSuccess(string sUserLogin, string sFromIP, int nFromPort, int nRegId)
        {
            cVaxRegUser objVaxRegUser = m_objVaxRegUserList.GetRegUserData(sUserLogin);

            if(objVaxRegUser == null)
                return;
            
            SetUserRegistered(sUserLogin);
            objVaxRegUser.OnRegisterUserSuccess(sUserLogin, sFromIP, nFromPort, nRegId);
        }

        public override void OnRegisterUserFailed(string sUserLogin, string sFromIP, int nFromPort, int nRegId)
        {
            cVaxRegUser objVaxRegUser = m_objVaxRegUserList.GetRegUserData(sUserLogin);

            if(objVaxRegUser == null)
                return;

            objVaxRegUser.OnRegisterUserFailed(sUserLogin, sFromIP, nFromPort, nRegId);
            m_objVaxRegUserList.RemoveRegUserData(sUserLogin);
        }

        public override void OnUnRegisterUser(string sUserLogin)
        {
            cVaxRegUser objVaxRegUser = m_objVaxRegUserList.GetRegUserData(sUserLogin);

            if(objVaxRegUser == null)
                return;

           objVaxRegUser.OnUnRegisterUser(sUserLogin);
           m_objVaxRegUserList.RemoveRegUserData(sUserLogin);
        }

        ///////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public override void OnChatMessageText(int nChatMsgId, string sMsgFrom, string sMsgTo, string sMsgText, int nFromPeerType, string sFromPeerName, string sFromIP, int nFromPort)
        {
            m_objChat.OnChatMessageText(nChatMsgId, sMsgFrom, sMsgTo, sMsgText, nFromPeerType, sFromPeerName, sFromIP, nFromPort);
        }
        public override void OnChatMessageTyping(int nChatMsgId, string sMsgFrom, string sMsgTo, bool bIsTypingStart, int nFromPeerType, string sFromPeerName, string sFromIP, int nFromPort)
        {
            m_objChat.OnChatMessageTyping(nChatMsgId, sMsgFrom, sMsgTo, bIsTypingStart, nFromPeerType, sFromPeerName, sFromIP, nFromPort);
        }
        public override void OnChatStatusSubscribe(int nSubscrbId, string sMsgFrom, string sMsgTo, int nFromPeerType, string sFromPeerName, string sFromIP, int nFromPort)
        {
            m_objChat.OnChatStatusSubscribe(nSubscrbId, sMsgFrom, sMsgTo, nFromPeerType, sFromPeerName, sFromIP, nFromPort);
        }

        public override void OnChatMessageSuccess(int nChatMsgId)
        {
            m_objChat.OnChatMessageSuccess(nChatMsgId);
        }
        public override void OnChatMessageFailed(int nChatMsgId, int nStatusCode, string sReasonPhrase)
        {
            m_objChat.OnChatMessageFailed(nChatMsgId, nStatusCode, sReasonPhrase);
        }
        public override void OnChatMessageTimeout(int nChatMsgId)
        {
            m_objChat.OnChatMessageTimeout(nChatMsgId);
        }
        
        public override void OnAddCallSessionToQueueSuccess(string sQueueName, int nSessionId)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);

            if(objCallSession == null)
                return;

            objCallSession.OnAddCallSessionToQueueSuccess(sQueueName, nSessionId);
        }

        public override void OnAddCallSessionToQueueFailed(string sQueueName, int nSessionId)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);

            if(objCallSession == null)
                return;

            objCallSession.OnAddCallSessionToQueueFailed(sQueueName, nSessionId);
        }

        public override void OnQueuePlayWaveStart(int nEventType, string sQueueName, int nSessionId)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);

            if(objCallSession == null)
                return;

            objCallSession.OnQueuePlayWaveStart(nEventType, sQueueName, nSessionId);
        }

        public override void OnQueuePlayWaveStop(int nEventType, string sQueueName, int nSessionId)
        {
           cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);

            if(objCallSession == null)
                return;

            objCallSession.OnQueuePlayWaveStop(nEventType, sQueueName, nSessionId);
        } 
        
        public override void OnAddCallSessionToRingGroupSuccess(string sRingGroupName, int nSessionId)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);

            if(objCallSession == null)
                return;

            objCallSession.OnAddCallSessionToRingGroupSuccess(nSessionId, sRingGroupName);
        }

        public override void OnAddCallSessionToRingGroupFailed(string sRingGroupName, int nSessionId)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);

            if(objCallSession == null)
                return;

            objCallSession.OnAddCallSessionToRingGroupFailed(nSessionId, sRingGroupName);
        }

        public override void OnVaxLicenseChannel(int nTotalChannels, int nUsedChannels)
        {

        }

       //////////////////////////////////////////////////////////////////////////////////
       //////////////////////////////////////////////////////////////////////////////////

        public override void OnBusyLampSubscribe(int nSubScrbId, string sUserName, string sToUserName, string sFromIP, int nFromPort)
        {
            base.BusyLampSubscribeAccept(nSubScrbId, true, 30);
        }

        public override void OnBusyLampSendStatus(string sFromUserName, string sToUserName, int nStateId)
        {
            base.BusyLampSendStatus(sFromUserName, sToUserName, nStateId);
        }

        public override void OnBusyLampUnSubscribe(int nSubScrbId, string sUserName, string sToUserName, string sFromIP, int nFromPort)
        {

        }

        public override void OnBusyLampSubscribeSuccess(int nSubScrbId, string sUserName)
        {

        }

        public override void OnBusyLampSubscribeFailed(int nSubScrbId, string sUserName)
        {

        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public void GenStatusLog(string sStatus)
        {
            OnStatusLog(sStatus);
        }

        public override void OnVaxErrorLog(string sFuncName, int nErrorCode, string sErrorMsg)
        {
             string sMsg = "[Method: " + sFuncName + "] [Error Msg: " + sErrorMsg + "] [Code: " + nErrorCode.ToString() + "]";
             OnErrorLog(sMsg);
        }

        public virtual void OnStatusLog(string sMsg)
        {

        }

        public virtual void OnErrorLog(string sMsg)
        {

        }

        public virtual void OnInitialize()
        {

        }

        #endregion

    }
}
