﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ServerSIP
{
    class cStoreDirectProxy : cStoreRegistry
    {
        const string PROXY_ROOT_PATH = "DirectProxies\\";

        public cStoreDirectProxy()
        {
        }

        ~cStoreDirectProxy()
        {
        }

        /////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////

        public bool AddDirectProxySIP(string sProxyName, string sAuthLogin, string sAuthPwd, string sDomainRealm, string sProxyIP, int nProxyPort, string sCodecList)
        {
            if(sProxyName.Length == 0)
                return false;

            string sKeyPath = PROXY_ROOT_PATH + sProxyName;

            base.SetTEXT(sKeyPath, "AuthLogin", sAuthLogin);
            base.SetTEXT(sKeyPath, "AuthPwd", sAuthPwd);
            base.SetTEXT(sKeyPath, "DomainRealm", sDomainRealm);
            
            base.SetTEXT(sKeyPath, "ProxyIP", sProxyIP);
            base.SetINT(sKeyPath, "ProxyPort", nProxyPort);
            base.SetTEXT(sKeyPath, "CodecList", sCodecList);

            return true;
        }

        public bool GetDirectProxySIP(string sProxyName, ref string sAuthLogin, ref string sAuthPwd, ref string sDomainRealm, ref string sProxyIP, ref int nProxyPort, ref string sCodecList)
        {
            if(sProxyName.Length == 0)
                return false;

            string sKeyPath = PROXY_ROOT_PATH + sProxyName;

            base.GetTEXT(sKeyPath, "AuthLogin", ref sAuthLogin);
            base.GetTEXT(sKeyPath, "AuthPwd", ref sAuthPwd);
            base.GetTEXT(sKeyPath, "DomainRealm", ref sDomainRealm);
            
            base.GetTEXT(sKeyPath, "ProxyIP", ref sProxyIP);
            base.GetINT(sKeyPath, "ProxyPort", ref nProxyPort);
            base.GetTEXT(sKeyPath, "CodecList", ref sCodecList);

            return true;
        }

        public void RemoveDirectProxySIP(string sProxyName)
        {
            if(sProxyName.Length == 0)
                return;

            string sKeyPath = PROXY_ROOT_PATH + sProxyName;
            base.DeleteSubKey(sKeyPath);
        }

        public bool GetDirectProxySIPAll(ref List<string> objListDirectProxies)
        {
            objListDirectProxies.Clear();
            string sKeyPath = PROXY_ROOT_PATH;
            return base.EnumSubKeys(sKeyPath, ref objListDirectProxies);
        }

        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////

        public bool AddDirectProxyRoute(string sProxyName, int nToPeerType, string sToPeerName, bool bRejected, string sRoomName, string sQueueName, string sRingGroupName)
        {
            if(sProxyName.Length == 0)
                return false;

            string sKeyPath = PROXY_ROOT_PATH + sProxyName;

            base.SetINT(sKeyPath, "ToPeerType", nToPeerType);
            base.SetBOOL(sKeyPath, "Rejected", bRejected);

            base.SetTEXT(sKeyPath, "ToPeerName", sToPeerName);
            base.SetTEXT(sKeyPath, "RoomName", sRoomName);

            base.SetTEXT(sKeyPath, "QueueName", sQueueName);
            base.SetTEXT(sKeyPath, "RingGroupName", sRingGroupName);

            return true;
        }

        public bool GetDirectProxyRoute(string sProxyName, ref int nToPeerType, ref string sToPeerName, ref bool bRejected, ref string sRoomName, ref string sQueueName, ref string sRingGroupName)
        {
            if(sProxyName.Length == 0)
                return false;

            string sKeyPath = PROXY_ROOT_PATH + sProxyName;

            base.GetINT(sKeyPath, "ToPeerType", ref nToPeerType);
            base.GetBOOL(sKeyPath, "Rejected", ref bRejected);

            base.GetTEXT(sKeyPath, "ToPeerName", ref sToPeerName);
            base.GetTEXT(sKeyPath, "RoomName", ref sRoomName);

            base.GetTEXT(sKeyPath, "QueueName", ref sQueueName);
            base.GetTEXT(sKeyPath, "RingGroupName", ref sRingGroupName);

            return true;
        }

        public bool RemoveDirectProxyRoute(string sProxyName)
        {
            if(sProxyName.Length == 0)
                return false;

            string sKeyPath = PROXY_ROOT_PATH + sProxyName;

            base.DeleteValue(sKeyPath, "ToPeerType");
            base.DeleteValue(sKeyPath, "Rejected");

            base.DeleteValue(sKeyPath, "ToPeerName");
            base.DeleteValue(sKeyPath, "RoomName");

            base.DeleteValue(sKeyPath, "QueueName");
            base.DeleteValue(sKeyPath, "RingGroupName");

            return true;
        }

        public bool IsDataStored()
        {
            return base.IsKeyExist(PROXY_ROOT_PATH);
        }

    }
}
